#! /usr/bin/perl
#
# v0.42 17.08.07 Peter Gastinger
#
# This script generate "secure" passwords and replaces
# the default passwords in following files:
#   /etc/squid/squid.conf
#   /etc/saslauthd.conf
#   /etc/freeradius/ldap.conf
#   /etc/freeradius/clients.conf
#   /etc/radiusclient/servers
#   /etc/chilli.conf
#   /etc/gibUser.conf
#   /etc/puresight/rootcn.conf
#   /etc/puresight/puresightcn.conf
#   /etc/ldap/slapd.conf
# and additionaly the passwords in the LDAP DIT.
#
# ./setpasswords -p
#   prints the new passwords
#
# --------------------------------------------------------------------------
# Copyright notice
# --------------------------------------------------------------------------
# Copyright: Peter Gastinger, Jan 2007
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL'.
# --------------------------------------------------------------------------
#
 
use strict; 
use Net::LDAP;
use Digest::SHA1;
use MIME::Base64;
use Getopt::Std;
 
my %opt;
getopts('p', \%opt);

my $admindn="ou=admin";
my $basedn="dc=gibraltar,dc=local";
my $ldapserver="localhost";
my $ldapport=389;
my $rootdn="uid=manager,".$admindn.",".$basedn;
my $rootpw="secret";

my @filestoadd = ("/etc/squid/squid.conf","/etc/saslauthd.conf","/etc/freeradius/ldap.conf","/etc/freeradius/clients.conf","/etc/radiusclient/servers","/etc/chilli.conf","/etc/gibUser.conf","/etc/puresight/rootcn.conf","/etc/puresight/puresightcn.conf","/etc/ldap/slapd.conf");
my @file;
my ($line,$ldap,$mesg,$i,$j);

my $newmanagerpwd=randomPassword(12);
my $newnassecret=randomPassword(12);
my $newsquidpwd=randomPassword(12);
my $newpostfixpwd=randomPassword(12);
my $newradiuspwd=randomPassword(12);
my $uamsecret=randomPassword(12);
my $puresightpw=randomPassword(12);

my $runningslapd = 0;
# start ldap server and wait 2 seconds
if (system("/etc/init.d/slapd status") != 0) { 
  system("/etc/init.d/slapd start");
  sleep(2);
  $runningslapd = 1;
}

# check whether the manager credentials are ok
  $ldap = Net::LDAP->new($ldapserver,port=>$ldapport,onerror=>'die') or die $@;
  $mesg=$ldap->bind($rootdn,password=>$rootpw,version=>3) or warn $@;
  $ldap->unbind();

# print passwords
if ($opt{'p'}) {
  print "managerpw: ".$newmanagerpwd."\n";
  print "nassecret: ".$newnassecret."\n";
  print "squidpw: ".$newsquidpwd."\n";
  print "postfixpw: ".$newpostfixpwd."\n";
  print "radiuspw: ".$newradiuspwd."\n";
}

## DON'T EDIT BELOW THIS LINE ##
## -------------------------- ##

for ($j=0;$j<@filestoadd;$j++) {
 open(FILE,$filestoadd[$j]) || die("Couldn't open file");
 @file=<FILE>;
 close(FILE);
 $i=0;
 open(FILE,">".$filestoadd[$j]) || die("Couldn't open file"); 
 foreach (@file) {
# The following regex matches squid.conf
   if ($file[$i] =~ /-D\s"uid=sadmin,ou=admin,dc=gibraltar,dc=local"\s-w\s"(.*)"\s-b/) {
     $file[$i] =~ s/$1/$newsquidpwd/g;
     changeldappw("sadmin",$newsquidpwd);
   }
# The following regex matches saslauthd (postfix)
   if ($file[$i] =~ /ldap_bind_pw:\s*(.*)/) {
     $file[$i] =~ s/$1/$newpostfixpwd/g;	
     changeldappw("padmin",$newpostfixpwd);
  }
# The following regex matches radiusd.conf
   if ($file[$i] =~/password\s=\s"(.*)"$/) {
     $file[$i] =~ s/$1/$newradiuspwd/g;
     changeldappw("radmin",$newradiuspwd);
   }
# The following regex matches clients.conf (radius)
   if ($file[$i] =~/secret\s*= (.*)$/) {
     $file[$i] =~ s/$1/$newnassecret/g;
   }
# The following regex matches servers (radiusclient)
   if ($file[$i] =~/^localhost\s*(.*)$/) {
     $file[$i] =~ s/$1/$newnassecret/g;
   }
# The following regex matches chilli
   if ($file[$i] =~/^radiussecret\s(.*)$/) {
     $file[$i] =~ s/$1/$newnassecret/g;
   }
# The following regex matches chilli
   if ($file[$i] =~/^uamsecret\s(.*)$/) {
     $file[$i] =~ s/$1/$uamsecret/g;
   }   
# The following regex matches gibUser.conf
   if ($file[$i] =~/^our\s\$rootpwd="(.*)"/) {
     $file[$i] =~ s/$1/$newmanagerpwd/g;
     changeldappw("manager",$newmanagerpwd);
   }
   
# The following regex matches gibUser.conf (used for keeping local pw)   
   if ($file[$i] =~/localrootpwd="(.*)"/) {
     $file[$i] =~ s/$1/$newmanagerpwd/g;
   }
   if ($file[$i] =~/localsadmin="(.*)"/) {
     $file[$i] =~ s/$1/$newsquidpwd/g;
   }
   if ($file[$i] =~/localpadmin="(.*)"/) {
     $file[$i] =~ s/$1/$newpostfixpwd/g;
   }
   if ($file[$i] =~/localradmin="(.*)"/) {
     $file[$i] =~ s/$1/$newradiuspwd/g;
   } 
      
# The following  matches PureSight files
  if (($filestoadd[$j] =~ /rootcn.conf/ || $filestoadd[$j] =~ /puresightcn.conf/) && $file[$i] =~ /^(secret)$/) {
     $file[$i] =~ s/$1/$puresightpw/g;	
  }
  if ($filestoadd[$j] =~ /slapd.conf/ && $file[$i] =~ /^rootpw\s*(secret)$/) {
     $file[$i] =~ s/$1/$puresightpw/g;	
  }
 print FILE $file[$i++];
 }
 close(FILE);
}

# stop ldap server
if ($runningslapd == 1) {
  system("/etc/init.d/slapd stop");
}

sub changeldappw{
  my($adminuid,$password)=@_;
  my %changeuserdata;
  my ($salt,$dn,$ctx,$ldap,$hashedPasswd);
  my @a;
# connect to LDAP
  $ldap = Net::LDAP->new($ldapserver,port=>$ldapport) or die $@;
  $ldap->bind($rootdn,password=>$rootpw,version=>3) or die $@;
  $dn="uid=".$adminuid.",".$admindn.",".$basedn;

# generate SALT
  srand (time ^ $$);
  @a = grep {/[a-zA-Z0-9]/} map {chr($_)} (0..256);
  for (my $m=0;$m<8;$m++) {
    $salt .= $a[ rand( @a ) ];
  }
# calculate ssha password
  $ctx = Digest::SHA1->new;
  $ctx->add($password);
  $ctx->add($salt);
  $hashedPasswd = '{SSHA}' . encode_base64($ctx->digest . $salt ,'');

  $changeuserdata{"uid"}=$adminuid;
  $changeuserdata{"userpassword"}=$hashedPasswd;

  $mesg = $ldap->modify($dn,
  replace => { %changeuserdata }
  );
  $mesg->code() && warn "Couldn't modify: ".$mesg->error;

  $ldap->unbind();
}

sub randomPassword {
  my @password;
  my @numbers = split(" ","1 2 3 4 5 6 7 8 9 0");
  my @special = split(" ",", . ; : _ + ?");
  my @lowerchars = split(" ","a b c d e f g h i j k l m n o p q r s t u v w x y z");
  my @upperchars = split(" ","A B C D E F G H I J K L M N O P Q R S T U V W X Y Z");
  my @chars = (@lowerchars,@upperchars);

  my $password_length = $_[0];
  if (!$password_length || $password_length < 10) {
    $password_length = 10;
  }

  my $countspecialchars = int($password_length/4);
  my $countnumbers = int($password_length/4);
  my $countchars = $password_length - $countspecialchars - $countnumbers;

  srand;
  for (my $i=0; $i < $countspecialchars  ;$i++) {
    @password[$i] = $special[int(rand @special)];
  }
  for (my $i=0; $i < $countnumbers  ;$i++) {
    $password[$countspecialchars+$i] = $numbers[int(rand @numbers)];
  }
  for (my $i=0; $i < $countchars ;$i++) {
    $password[$password_length-$countchars+$i] = $chars[int (rand @chars)];
  }
  
  fisher_yates_shuffle( \@password ); 
  return join("",@password);
}
   
# shuffle password array
sub fisher_yates_shuffle {
    my $array = shift;
    my $i;
    for ($i = @$array; --$i; ) {
        my $j = int rand ($i+1);
        next if $i == $j;
        @$array[$i,$j] = @$array[$j,$i];
    }
}
